Package: pathwayPCA
Type: Package
Title: Integrative Pathway Analysis with Modern PCA Methodology and
        Gene Selection
Version: 1.28.0
Authors@R: c(person("Gabriel", "Odom", email = "gabriel.odom@med.miami.edu", role = c("aut","cre")),
  person("James", "Ban", email = "yuguang.ban@med.miami.edu", role = c("aut")),
  person("Lizhong", "Liu", email = "lxl816@miami.edu", role = c("aut")),
  person("Lily", "Wang", email = "lily.wang@med.miami.edu", role = c("aut")),
  person("Steven", "Chen", email = "steven.chen@med.miami.edu", role = c("aut")))
Description: pathwayPCA is an integrative analysis tool that implements
        the principal component analysis (PCA) based pathway analysis
        approaches described in Chen et al. (2008), Chen et al. (2010),
        and Chen (2011). pathwayPCA allows users to: (1) Test pathway
        association with binary, continuous, or survival phenotypes.
        (2) Extract relevant genes in the pathways using the SuperPCA
        and AES-PCA approaches. (3) Compute principal components (PCs)
        based on the selected genes. These estimated latent variables
        represent pathway activities for individual subjects, which can
        then be used to perform integrative pathway analysis, such as
        multi-omics analysis. (4) Extract relevant genes that drive
        pathway significance as well as data corresponding to these
        relevant genes for additional in-depth analysis. (5) Perform
        analyses with enhanced computational efficiency with parallel
        computing and enhanced data safety with S4-class data objects.
        (6) Analyze studies with complex experimental designs, with
        multiple covariates, and with interaction effects, e.g.,
        testing whether pathway association with clinical phenotype is
        different between male and female subjects. Citations: Chen et
        al. (2008) <https://doi.org/10.1093/bioinformatics/btn458>;
        Chen et al. (2010) <https://doi.org/10.1002/gepi.20532>; and
        Chen (2011) <https://doi.org/10.2202/1544-6115.1697>.
License: GPL-3
Depends: R (>= 3.1)
Imports: lars, methods, parallel, stats, survival, utils
Suggests: airway, circlize, grDevices, knitr, RCurl, reshape2,
        rmarkdown, SummarizedExperiment, survminer, testthat, tidyverse
biocViews: CopyNumberVariation, DNAMethylation, GeneExpression, SNP,
        Transcription, GenePrediction, GeneSetEnrichment,
        GeneSignaling, GeneTarget, GenomeWideAssociation,
        GenomicVariation, CellBiology, Epigenetics, FunctionalGenomics,
        Genetics, Lipidomics, Metabolomics, Proteomics, SystemsBiology,
        Transcriptomics, Classification, DimensionReduction,
        FeatureExtraction, PrincipalComponent, Regression, Survival,
        MultipleComparison, Pathways
Encoding: UTF-8
LazyData: false
RoxygenNote: 7.2.3
Collate: 'CreatePathwayCollection.R' 'createClass_OmicsPath.R'
        'createClass_validOmics.R' 'accessClass_OmicsPath.R'
        'createClass_OmicsSurv.R' 'accessClass_OmicsSurv.R'
        'accessClass_OmicsRegCateg.R' 'createClass_OmicsCateg.R'
        'createClass_OmicsReg.R' 'accessClass_OmicsPathData.R'
        'accessClass_pathwayCollection.R'
        'accessClass_pathwayCollection_which.R' 'accessClass_pcOut.R'
        'accessClass_pcOutpVals.R' 'aesPC_calculate_AESPCA.R'
        'aesPC_calculate_LARS.R' 'aesPC_extract_OmicsPath_PCs.R'
        'aesPC_permtest_CoxPH.R' 'aesPC_permtest_GLM.R'
        'aesPC_permtest_LM.R' 'aesPC_unknown_matrixNorm.R'
        'aesPC_wrapper.R' 'createOmics_All.R'
        'createOmics_CheckAssay.R'
        'createOmics_CheckPathwayCollection.R'
        'createOmics_CheckSampleIDs.R' 'createOmics_JoinPhenoAssay.R'
        'createOmics_TrimPathwayCollection.R' 'createOmics_Wrapper.R'
        'data_colonSubset.R' 'data_genesetSubset.R'
        'data_wikipathways.R' 'data_wikipathways_symbols.R'
        'pathwayPCA.R' 'printClass_Omics_All.R'
        'printClass_pathwayCollection.R' 'superPC_model_CoxPH.R'
        'superPC_model_GLM.R' 'superPC_model_LS.R'
        'superPC_model_tStats.R' 'superPC_model_train.R'
        'superPC_modifiedSVD.R' 'superPC_optimWeibullParams.R'
        'superPC_optimWeibull_pValues.R' 'superPC_pathway_tControl.R'
        'superPC_pathway_tScores.R' 'superPC_pathway_tValues.R'
        'superPC_permuteSamples.R' 'superPC_wrapper.R'
        'utils_Contains.R' 'utils_adjust_and_sort_pValues.R'
        'utils_load_test_data_onto_PCs.R' 'utils_multtest_pvalues.R'
        'utils_read_gmt.R' 'utils_stdExpr_2_tidyAssay.R'
        'utils_transpose_assay.R' 'utils_write_gmt.R'
VignetteBuilder: knitr
URL: <https://gabrielodom.github.io/pathwayPCA/>
BugReports: https://github.com/gabrielodom/pathwayPCA/issues
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:49:48 UTC
RemoteUrl: https://github.com/bioc/pathwayPCA
RemoteRef: RELEASE_3_23
RemoteSha: a321cbd5f6babd08348464b13be711a9942eac14
NeedsCompilation: no
Packaged: 2026-04-29 23:48:00 UTC; root
Author: Gabriel Odom [aut, cre],
  James Ban [aut],
  Lizhong Liu [aut],
  Lily Wang [aut],
  Steven Chen [aut]
Maintainer: Gabriel Odom <gabriel.odom@med.miami.edu>
Built: R 4.6.0; ; 2026-04-30 01:20:51 UTC; unix
