# Generated by roxygen2: do not edit by hand

S3method("[[",pathwayCollection)
S3method(getPathPCLs,aespcOut)
S3method(getPathPCLs,superpcOut)
S3method(getPathpVals,aespcOut)
S3method(getPathpVals,superpcOut)
S3method(print,pathwayCollection)
export("getAssay<-")
export("getEvent<-")
export("getEventTime<-")
export("getPathwayCollection<-")
export("getResponse<-")
export("getSampleIDs<-")
export(AESPCA_pVals)
export(Contains)
export(CreateOmics)
export(CreatePathwayCollection)
export(ExtractAESPCs)
export(LoadOntoPCs)
export(SE2Tidy)
export(SubsetPathwayData)
export(SuperPCA_pVals)
export(TransposeAssay)
export(WhichPathways)
export(aespca)
export(getAssay)
export(getEvent)
export(getEventTime)
export(getPathPCLs)
export(getPathpVals)
export(getPathwayCollection)
export(getResponse)
export(getSampleIDs)
export(getTrimPathwayCollection)
export(read_gmt)
export(write_gmt)
exportClasses(OmicsCateg)
exportClasses(OmicsPathway)
exportClasses(OmicsReg)
exportClasses(OmicsSurv)
exportMethods(show)
importFrom(lars,backsolvet)
importFrom(lars,downdateR)
importFrom(lars,updateR)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,setClass)
importFrom(methods,setGeneric)
importFrom(methods,setMethod)
importFrom(methods,setOldClass)
importFrom(methods,show)
importFrom(methods,slot)
importFrom(methods,validObject)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,clusterExport)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallel,parLapply)
importFrom(parallel,parLapplyLB)
importFrom(parallel,parSapply)
importFrom(parallel,stopCluster)
importFrom(stats,AIC)
importFrom(stats,anova)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,deriv)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,optim)
importFrom(stats,pf)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,rweibull)
importFrom(stats,sd)
importFrom(survival,Surv)
importFrom(survival,coxph)
importFrom(survival,coxph.control)
importFrom(survival,is.Surv)
importFrom(survival,survreg)
importFrom(utils,str)
