#'  Colon Cancer -Omics Data
#'
#' @description Subset of a colon cancer survival data set, with subject
#'   response and assay values.
#'
#' @format A subset of a data frame containing 656 of 2022 genes measured on
#'   250 subjects. The first two columns are the Overall Survival time
#'   (\code{OS_time}) and death indicator (\code{OS_event}).
#'
#' @source GEO GSE17538
#'   \url{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE17538}
#' @usage data(colonSurv_df)
"colonSurv_df"
